#!/bin/bash
#Description: DevKit constants
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-2-9 created

function init_variable() {
	if [ ! -f "${INSTALL_PATH}/tools/log.sh" ]; then
	    echo "There is no log file(${INSTALL_PATH}/tools/log.sh), please check!"
		exit "${ERROR}"
	fi
	source ${INSTALL_PATH}/tools/log.sh
	INSTALL_INI_PATH="${INSTALL_PATH}/config/install.ini"
	if [[ ! -f "${INSTALL_INI_PATH}" ]]; then
		echo_and_log "There is no configure file(${INSTALL_INI_PATH}), please check!" "${ERROR}"
		exit "${ERROR}"
	fi
	USER_PATH="$(grep -w USER_PATH "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
	USER_NAME="$(grep -w USER_NAME "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
    os_type=$(grep -w OS_TYPE "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
    os_version=$(grep -w OS_VERSION "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
    plugin_name="affinity"
	INSTALL_LOG_PATH="${INSTALL_PATH}/logs/install_plugins.log"
	INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins"
	# 插件压缩包存档路径
	PYTHON_PATH="${INSTALL_PATH}/tools/python_venv/bin/python3"
}

function change_file_permission() {
    local file="$1"
    local num="$2"
    if [ -f "${file}" ]; then
        chmod "${num}" "${file}"
    fi
}

# 比较系统中与工具自带的libstd++.so.6版本大小
function compare_libstdc_version(){
    libstdc_version_of_tools="6.0.24"
    path_list=(/usr/lib64/ /usr/lib/ /lib64/ /lib /libx32)
    for path in "${path_list[@]}"; do
        if [[ -d $path ]]; then
            result=$(find $path -name libstdc++.so.6.*)
            if [[ -n "$result" ]]; then
                libstdc_version_of_system=$(echo "$result" | tr " " "\n" | sort -rV | head -n 1 | grep -Po "\d+\.\d+\.\d+")
                if version_gt "$libstdc_version_of_tools" "$libstdc_version_of_system"; then
                    libstdcflag="true"
                else
                    libstdcflag="false"
                fi
                break
            fi
        fi
    done
}

# 比较版本号字符串大小
function version_gt(){
    test "$(echo $@ | tr " " "\n" | sort -rV | head -n 1)" == "$1"
}

function add_ld_library_path() {
	#判断是x86平台还是arm平台
	platform=$(uname -m)
	if [ "${platform}" == "x86_64" ]; then
		return 0
	fi
	cpu_implementers=(0x48 0x41) # 鲲鹏平台特征值. 两个值必须同时存在例如0x48和0xd01
	cpu_parts=(0xd01 0xd08)
	# 只要cpu配置配置在列表中就支持内存一致性
	# WeakConsistency弱内存拷贝
	cpu_implementer=$(tail /proc/cpuinfo | grep "CPU implementer" | awk '{print $4}')
	cpu_part=$(tail /proc/cpuinfo | grep "CPU part" | awk '{print $4}')
	if [[ "${cpu_parts[@]}" =~ "${cpu_part}" ]] && [[ "${cpu_implementers[@]}" =~ "${cpu_implementer}" ]]; then
        # WeakConsistency弱内存拷贝
		if [[ "$libstdcflag" == "true" || -z "$libstdcflag" ]]; then
			mv -f ${INSTALL_PLUGIN_PATH}/${plugin_name}/tools/weakconsistency/staticcodeanalyzer/libstdc++.so.6* /${USER_PATH}/lib
			chmod 600 ${USER_PATH}/lib/*
			chown ${USER_NAME}:${USER_NAME} ${USER_PATH}/lib/libstdc++.so.6*
		fi
	fi
}

function main() {
	INSTALL_PATH="$1" # /opt/DevKit
	init_variable
	compare_libstdc_version
	add_ld_library_path
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/acceleration_library.json.bz2.cipher" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/log.conf" 600
}

main $@